@@ -1,37 +1,42 @@
+from __future__ import absolute_import
+
 import glob
 import logging
 import os
-import sys
-import traceback
-from urllib import pathname2url, unquote
-import urlparse
 import signal
-
+import sys
 import time
+import traceback
+import urlparse
+from urllib import pathname2url, unquote
+
 from PyQt5 import uic
+from PyQt5.QtCore import QCoreApplication, QObject, QPoint, QSettings, QStringListModel, QTimer, QUrl, Qt, \
+    pyqtSignal, pyqtSlot
 from PyQt5.QtCore import QDir
-from PyQt5.QtCore import Qt, pyqtSignal, QStringListModel, QSettings, QPoint, QCoreApplication, pyqtSlot, QUrl, \
-    QObject, QTimer
-from PyQt5.QtGui import QIcon, QDesktopServices
+from PyQt5.QtGui import QDesktopServices, QIcon
 from PyQt5.QtGui import QKeySequence
 from PyQt5.QtGui import QPixmap
-from PyQt5.QtWidgets import QMainWindow, QLineEdit, QTreeWidget, QSystemTrayIcon, QAction, QFileDialog, \
-    QCompleter, QApplication, QStyledItemDelegate, QListWidget
+from PyQt5.QtWidgets import QAction, QApplication, QCompleter, QFileDialog, QLineEdit, QListWidget, QMainWindow, \
+    QStyledItemDelegate, QSystemTrayIcon, QTreeWidget
 from PyQt5.QtWidgets import QShortcut
 
+import six
+
 from Tribler.Core.Modules.process_checker import ProcessChecker
-from TriblerGUI.tribler_action_menu import TriblerActionMenu
+
 from TriblerGUI.core_manager import CoreManager
 from TriblerGUI.debug_window import DebugWindow
-from TriblerGUI.defs import PAGE_SEARCH_RESULTS, \
-    PAGE_HOME, PAGE_EDIT_CHANNEL, PAGE_VIDEO_PLAYER, PAGE_DOWNLOADS, PAGE_SETTINGS, PAGE_SUBSCRIBED_CHANNELS, \
-    PAGE_CHANNEL_DETAILS, PAGE_PLAYLIST_DETAILS, BUTTON_TYPE_NORMAL, BUTTON_TYPE_CONFIRM, PAGE_LOADING, \
-    PAGE_DISCOVERING, PAGE_DISCOVERED, PAGE_TRUST, SHUTDOWN_WAITING_PERIOD, DEFAULT_API_PORT
+from TriblerGUI.defs import BUTTON_TYPE_CONFIRM, BUTTON_TYPE_NORMAL, DEFAULT_API_PORT, PAGE_CHANNEL_DETAILS, \
+    PAGE_DISCOVERED, PAGE_DISCOVERING, PAGE_DOWNLOADS, PAGE_EDIT_CHANNEL, PAGE_HOME, PAGE_LOADING, \
+    PAGE_PLAYLIST_DETAILS, PAGE_SEARCH_RESULTS, PAGE_SETTINGS, PAGE_SUBSCRIBED_CHANNELS, PAGE_TRUST, \
+    PAGE_VIDEO_PLAYER, SHUTDOWN_WAITING_PERIOD
 from TriblerGUI.dialogs.confirmationdialog import ConfirmationDialog
 from TriblerGUI.dialogs.feedbackdialog import FeedbackDialog
 from TriblerGUI.dialogs.startdownloaddialog import StartDownloadDialog
-from TriblerGUI.tribler_request_manager import request_queue, TriblerRequestManager, dispatcher
-from TriblerGUI.utilities import get_ui_file_path, get_image_path, get_gui_setting, is_dir_writable, quote_plus_unicode
+from TriblerGUI.tribler_action_menu import TriblerActionMenu
+from TriblerGUI.tribler_request_manager import TriblerRequestManager, dispatcher, request_queue
+from TriblerGUI.utilities import get_gui_setting, get_image_path, get_ui_file_path, is_dir_writable, quote_plus_unicode
 
 # Pre-load form UI classes
 fc_channel_torrent_list_item, _ = uic.loadUiType(get_ui_file_path('channel_torrent_list_item.ui'))
@@ -54,6 +59,7 @@ class MagnetHandler(QObject):
 class TriblerWindow(QMainWindow):
     resize_event = pyqtSignal()
     escape_pressed = pyqtSignal()
+    tribler_crashed = pyqtSignal(str)
     received_search_completions = pyqtSignal(object)
 
     def on_exception(self, *exc_info):
@@ -63,6 +69,10 @@ class TriblerWindow(QMainWindow):
 
         self.exception_handler_called = True
 
+        exception_text = "".join(traceback.format_exception(*exc_info))
+        logging.error(exception_text)
+        self.tribler_crashed.emit(exception_text)
+
         self.delete_tray_icon()
 
         # Stop the download loop
@@ -79,9 +89,6 @@ class TriblerWindow(QMainWindow):
         if self.debug_window:
             self.debug_window.setHidden(True)
 
-        exception_text = "".join(traceback.format_exception(*exc_info))
-        logging.error(exception_text)
-
         dialog = FeedbackDialog(self, exception_text, self.core_manager.events_manager.tribler_version,
                                 self.start_time)
         dialog.show()
@@ -120,6 +127,7 @@ class TriblerWindow(QMainWindow):
         self.start_time = time.time()
         self.exception_handler_called = False
         self.token_refresh_timer = None
+        self.shutdown_timer = None
 
         sys.excepthook = self.on_exception
 
@@ -172,6 +180,7 @@ class TriblerWindow(QMainWindow):
         self.discovering_page.initialize_discovering_page()
         self.discovered_page.initialize_discovered_page()
         self.trust_page.initialize_trust_page()
+        self.token_mining_page.initialize_token_mining_page()
 
         self.stackedWidget.setCurrentIndex(PAGE_LOADING)
 
@@ -250,6 +259,7 @@ class TriblerWindow(QMainWindow):
         self.core_manager.events_manager.events_started.connect(self.on_events_started)
         self.core_manager.events_manager.low_storage_signal.connect(self.on_low_storage)
         self.core_manager.events_manager.credit_mining_signal.connect(self.on_credit_mining_error)
+        self.core_manager.events_manager.tribler_shutdown_signal.connect(self.on_tribler_shutdown_state_update)
 
         # Install signal handler for ctrl+c events
         def sigint_handler(*_):
@@ -270,7 +280,7 @@ class TriblerWindow(QMainWindow):
         self.show()
 
     def update_tray_icon(self, use_monochrome_icon):
-        if not QSystemTrayIcon.isSystemTrayAvailable():
+        if not QSystemTrayIcon.isSystemTrayAvailable() or not self.tray_icon:
             return
 
         if use_monochrome_icon:
@@ -417,7 +427,7 @@ class TriblerWindow(QMainWindow):
         # Save the download location to the GUI settings
         current_settings = get_gui_setting(self.gui_settings, "recent_download_locations", "")
         recent_locations = current_settings.split(",") if len(current_settings) > 0 else []
-        if isinstance(destination, unicode):
+        if isinstance(destination, six.text_type):
             destination = destination.encode('utf-8')
         encoded_destination = destination.encode('hex')
         if encoded_destination in recent_locations:
@@ -652,10 +662,10 @@ class TriblerWindow(QMainWindow):
                 ConfirmationDialog.show_error(self, "Tribler UI Error", "Something went wrong. Please try again.")
                 logging.exception("Error while trying to download. Either dialog or dialog.dialog_widget is None")
 
-        self.dialog.request_mgr.cancel_request()  # To abort the torrent info request
-        self.dialog.close_dialog()
-        self.dialog = None
-        self.start_download_dialog_active = False
+        if self.dialog:
+            self.dialog.close_dialog()
+            self.dialog = None
+            self.start_download_dialog_active = False
 
         if action == 0:  # We do this after removing the dialog since process_uri_request is blocking
             self.process_uri_request()
@@ -678,7 +688,7 @@ class TriblerWindow(QMainWindow):
     def on_confirm_add_directory_dialog(self, action):
         if action == 0:
             for torrent_file in self.selected_torrent_files:
-                escaped_uri = u"file:%s" % pathname2url(torrent_file)
+                escaped_uri = u"file:%s" % pathname2url(torrent_file.encode('utf-8'))
                 self.perform_start_download_request(escaped_uri,
                                                     self.window().tribler_settings['download_defaults'][
                                                          'anonymity_enabled'],
@@ -849,7 +859,8 @@ class TriblerWindow(QMainWindow):
     def resizeEvent(self, _):
         # Resize home page cells
         cell_width = self.home_page_table_view.width() / 3 - 3  # We have some padding to the right
-        cell_height = cell_width / 2 + 60
+        max_height = self.home_page_table_view.height() / 3 - 4
+        cell_height = min(cell_width / 2 + 60, max_height)
 
         for i in range(0, 3):
             self.home_page_table_view.setColumnWidth(i, cell_width)
@@ -865,9 +876,6 @@ class TriblerWindow(QMainWindow):
     def close_tribler(self):
         if not self.core_manager.shutting_down:
             def show_force_shutdown():
-                self.loading_text_label.setText("Tribler is taking longer than expected to shut down. You can force "
-                                                "Tribler to shutdown by pressing the button below. This might lead "
-                                                "to data loss.")
                 self.window().force_shutdown_btn.show()
 
             self.delete_tray_icon()
@@ -875,6 +883,9 @@ class TriblerWindow(QMainWindow):
             self.hide_status_bar()
             self.loading_text_label.setText("Shutting down...")
 
+            if self.debug_window:
+                self.debug_window.setHidden(True)
+
             self.shutdown_timer = QTimer()
             self.shutdown_timer.timeout.connect(show_force_shutdown)
             self.shutdown_timer.start(SHUTDOWN_WAITING_PERIOD)
@@ -931,6 +942,9 @@ class TriblerWindow(QMainWindow):
         # Stop the Qt application
         QApplication.quit()
 
+    def on_tribler_shutdown_state_update(self, state):
+        self.loading_text_label.setText(state)
+
 
 def _qurl_to_path(qurl):
     parsed = urlparse.urlparse(qurl.toString())
